#!/usr/bin/env node
const path = require('path');
const fs = require('fs');
const multer = require('multer');
const xlsx = require('node-xlsx');
const cors = require('cors');
const upload = multer({
	dest: 'upload/'
});
const express = require('express');
const valid = require('url-valid'); //验证是否有效
const validUrl = require('valid-url'); //验证是否为url
var allSettled = require('promise.allsettled');
const bodyParser = require('body-parser');
var app = express();
var http = require('http').createServer(app);
var io = require('socket.io')(http);
app.use(cors());
app.use(bodyParser.urlencoded({
	extended: true
}));
app.use(express.static(path.join(__dirname, 'view')));
app.use('/node_modules', express.static(path.join(__dirname, 'node_modules')))
io.on('connection', (socket) => {
	console.log('a user connected');
});
app.post('/checkUrl', upload.single('file'), (req, res) => {
	let invalidUrl = new Array();
	var file = req.file;
	var [sheets] = xlsx.parse(file.path); //解析文件
	const {
		data
	} = sheets;
	let linkArr = new Array();
	let spineArr = new Array();
	let hasName = data[0].findIndex((v) => {
		return String(v).includes('名称');
	})
	for (let i = 0, len = data.length; i < len; i++) {
		for (let j = 0, rowLen = data[i].length; j < rowLen; j++) {
			try {
				let xlsxItem = String(data[i][j]);
				if (validUrl.isUri(xlsxItem)) {
					linkArr.push(data[i][j]);
					let atlasFile = path.parse(xlsxItem);
					if (atlasFile.ext === '.atlas') {
						try {
							let spineAnim = {
								spineAtlas: xlsxItem,
								spineJson: findSpineFile(data[i], '.json', atlasFile),
								spinePng: findSpineFile(data[i], '.png', atlasFile),
								spineName: hasName === -1 ? atlasFile.name : data[i][hasName],
								spineId: String(i)
							}
							let relatedPictures = new Array();
							data[i].forEach((v,pIndex) => {
								if(path.extname(String(v)) === '.png' && v !== spineAnim.spinePng){
								 let picObj = {
									 name:data[0][pIndex],
									 pic:v
								 }	
								 relatedPictures.push(picObj)
								}
								
							})
							if (relatedPictures.length > 0) {
								spineAnim.relatedPictures = relatedPictures;
							}
							spineArr.push(spineAnim);
						} catch (e) {
							console.log(e)
						}
					}
				}
			} catch (error) {
				console.log(error)
			}

		}
	}
	if (linkArr.length === 0) {
		res.send();
		return
	}
	io.emit('linkArrLen', {
		len: linkArr.length,
		uid: req.body.uid
	});
	let hasValided = 0;
	Promise.all(linkArr.map(item => {
		return new Promise((reslove, reject) => {
			valid(item, (err, valid) => {

				if (!valid) {
					invalidUrl.push(item);
				}
				hasValided++;
				io.emit('updateProgress', {
					hasValided: hasValided,
					uid: req.body.uid
				});
				reslove();
			});
		});
	})).then(() => {
		res.json({
			invalidUrlArr: invalidUrl,
			spineArr: spineArr
		});
		fs.unlink(file.path, function(error) { //文件删除
			if (error) {
				console.log(error);
			}
		})

	});
})
const findSpineFile = (rowData, fileType, atlasFile) => {
	let spineFile = rowData.filter(v => {
		v = String(v);
		return path.extname(v) === fileType && path.parse(v).dir === atlasFile.dir && path.parse(v).name === atlasFile.name;
	})
	if (spineFile.length > 0) {
		return spineFile[0]
	} else {
		return null
	}
}
app.post('/uploadFolder',upload.single('file'),(req,res) => {
	
})
const [execPath,argfile,port] = process.argv;
let serverPort = port || 8080
http.listen(serverPort, () => {
	console.log(`listening on *:${serverPort}`);
});
